@@ -167,15 +167,11 @@ class EnvelopeBuffer(Buffer):
                 description += ', with key '
             elif len(encrypt_keys) > 1:
                 description += ', with keys '
-            first_key = True
+            key_ids = []
             for key in encrypt_keys:
-                if key is not None:
-                    if first_key:
-                        first_key = False
-                    else:
-                        description += ', '
-                    if len(key.subkeys) > 0:
-                        description += key.uids[0].uid
+                if key is not None and key.subkeys:
+                    key_ids.append(key.uids[0].uid)
+            description += ', '.join(key_ids)
             lines.append(('GPG encrypt', description))
 
         if self.envelope.tags:
@@ -257,7 +253,6 @@ class SearchBuffer(Buffer):
         self.reversed = reverse
         self.kill_filler_process()
 
-        self.result_count = self.dbman.count_messages(self.querystring)
         if reverse:
             order = self._REVERSE[self.sort_order]
         else:
@@ -268,6 +263,7 @@ class SearchBuffer(Buffer):
             exclude_tags = [t for t in exclude_tags.split(';') if t]
 
         try:
+            self.result_count = self.dbman.count_messages(self.querystring)
             self.pipe, self.proc = self.dbman.get_threads(self.querystring,
                                                           order,
                                                           exclude_tags)
@@ -341,6 +337,7 @@ class ThreadBuffer(Buffer):
         self._auto_unread_dont_touch_mids = set([])
         self._auto_unread_writing = False
 
+        self._indent_width = settings.get('thread_indent_replies')
         self.rebuild()
         Buffer.__init__(self, ui, self.body)
 
@@ -371,14 +368,29 @@ class ThreadBuffer(Buffer):
 
         self._tree = ThreadTree(self.thread)
 
-        bars_att = settings.get_theming_attribute('thread', 'arrow_bars')
-        heads_att = settings.get_theming_attribute('thread', 'arrow_heads')
-        A = ArrowTree(
-            self._tree,
-            indent=2,
-            childbar_offset=0,
-            arrow_tip_att=heads_att,
-            arrow_att=bars_att)
+        # define A to be the tree to be wrapped by a NestedTree and displayed.
+        # We wrap the thread tree into an ArrowTree for decoration if
+        # indentation was requested and otherwise use it as is.
+        if self._indent_width == 0:
+            A = self._tree
+        else:
+            # we want decoration.
+            bars_att = settings.get_theming_attribute('thread', 'arrow_bars')
+            # only add arrow heads if there is space (indent > 1).
+            heads_char = None
+            heads_att = None
+            if self._indent_width > 1:
+                heads_char = u'\u27a4'
+                heads_att = settings.get_theming_attribute('thread',
+                                                           'arrow_heads')
+            A = ArrowTree(
+                self._tree,
+                indent=self._indent_width,
+                childbar_offset=0,
+                arrow_tip_att=heads_att,
+                arrow_tip_char=heads_char,
+                arrow_att=bars_att)
+
         self._nested_tree = NestedTree(A, interpret_covered=True)
         self.body = TreeBox(self._nested_tree)
         self.message_count = self.thread.get_total_messages()
