@@ -12,7 +12,7 @@ import glob
 import logging
 import os
 import subprocess
-from StringIO import StringIO
+from io import StringIO
 
 import urwid
 from twisted.internet.defer import inlineCallbacks
@@ -35,6 +35,7 @@ from ..widgets.utils import DialogBox
 from ..db.errors import DatabaseLockedError
 from ..db.envelope import Envelope
 from ..settings.const import settings
+from ..settings.errors import ConfigError, NoMatchingAccount
 from ..utils import argparse as cargparse
 
 MODE = 'global'
@@ -78,7 +79,8 @@ class ExitCommand(Command):
 
         if ui.db_was_locked:
             msg = 'Database locked. Exit without saving?'
-            if (yield ui.choice(msg, msg_position='left', cancel='no')) == 'no':
+            response = yield ui.choice(msg, msg_position='left', cancel='no')
+            if response == 'no':
                 return
             ui.exit()
 
@@ -247,7 +249,7 @@ class ExternalCommand(Command):
         # set standard input for subcommand
         stdin = None
         if self.stdin is not None:
-            # wrap strings in StrinIO so that they behaves like a file
+            # wrap strings in StringIO so that they behave like files
             if isinstance(self.stdin, unicode):
                 stdin = StringIO(self.stdin)
             else:
@@ -255,7 +257,7 @@ class ExternalCommand(Command):
 
         def afterwards(data):
             if data == 'success':
-                if callable(self.on_success):
+                if self.on_success is not None:
                     self.on_success()
             else:
                 ui.notify(data, priority='error')
@@ -267,35 +269,23 @@ class ExternalCommand(Command):
 
         def thread_code(*_):
             try:
-                if stdin is None:
-                    proc = subprocess.Popen(self.cmdlist, shell=self.shell,
-                                            stderr=subprocess.PIPE)
-                    ret = proc.wait()
-                    err = proc.stderr.read()
-                else:
-                    proc = subprocess.Popen(self.cmdlist, shell=self.shell,
-                                            stdin=subprocess.PIPE,
-                                            stderr=subprocess.PIPE)
-                    _, err = proc.communicate(stdin.read())
-                    ret = proc.wait()
-                if ret == 0:
-                    return 'success'
-                else:
-                    return err.strip()
+                proc = subprocess.Popen(self.cmdlist, shell=self.shell,
+                                        stdin=subprocess.PIPE if stdin else None,
+                                        stderr=subprocess.PIPE)
             except OSError as e:
                 return str(e)
 
+            _, err = proc.communicate(stdin.read() if stdin else None)
+            if proc.returncode == 0:
+                return 'success'
+            return err.strip()
+
         if self.in_thread:
             d = threads.deferToThread(thread_code)
             d.addCallback(afterwards)
         else:
-            ui.mainloop.screen.stop()
-            ret = thread_code()
-            ui.mainloop.screen.start()
-
-            # make sure urwid renders its canvas at the correct size
-            ui.mainloop.screen_size = None
-            ui.mainloop.draw_screen()
+            with ui.paused():
+                ret = thread_code()
 
             afterwards(ret)
 
@@ -353,9 +343,8 @@ class PythonShellCommand(Command):
     repeatable = True
 
     def apply(self, ui):
-        ui.mainloop.screen.stop()
-        code.interact(local=locals())
-        ui.mainloop.screen.start()
+        with ui.paused():
+            code.interact(local=locals())
 
 
 @registerCommand(MODE, 'repeat')
@@ -680,6 +669,8 @@ class HelpCommand(Command):
     (['--sender'], {'nargs': '?', 'help': 'sender'}),
     (['--template'], {'nargs': '?',
                       'help': 'path to a template message file'}),
+    (['--tags'], {'nargs': '?',
+                  'help': 'comma-separated list of tags to apply to message'}),
     (['--subject'], {'nargs': '?', 'help': 'subject line'}),
     (['--to'], {'nargs': '+', 'help': 'recipients'}),
     (['--cc'], {'nargs': '+', 'help': 'copy to'}),
@@ -695,7 +686,7 @@ class ComposeCommand(Command):
 
     """compose a new email"""
     def __init__(self, envelope=None, headers=None, template=None, sender=u'',
-                 subject=u'', to=None, cc=None, bcc=None, attach=None,
+                 tags=None, subject=u'', to=None, cc=None, bcc=None, attach=None,
                  omit_signature=False, spawn=None, rest=None, encrypt=False,
                  **kwargs):
         """
@@ -709,6 +700,8 @@ class ComposeCommand(Command):
         :type template: str
         :param sender: From-header value
         :type sender: str
+        :param tags: Comma-separated list of tags to apply to message
+        :type tags: list(str)
         :param subject: Subject-header value
         :type subject: str
         :param to: To-header value
@@ -746,6 +739,7 @@ class ComposeCommand(Command):
         self.force_spawn = spawn
         self.rest = ' '.join(rest or [])
         self.encrypt = encrypt
+        self.tags = tags
 
     @inlineCallbacks
     def apply(self, ui):
@@ -761,11 +755,6 @@ class ComposeCommand(Command):
         if self.template is not None:
             # get location of tempsdir, containing msg templates
             tempdir = settings.get('template_dir')
-            tempdir = os.path.expanduser(tempdir)
-            if not tempdir:
-                xdgdir = os.environ.get('XDG_CONFIG_HOME',
-                                        os.path.expanduser('~/.config'))
-                tempdir = os.path.join(xdgdir, 'alot', 'templates')
 
             path = os.path.expanduser(self.template)
             if not os.path.dirname(path):  # use tempsdir
@@ -780,8 +769,11 @@ class ComposeCommand(Command):
                           priority='error')
                 return
             try:
-                with open(path) as f:
-                    self.envelope.parse_template(f.read())
+                with open(path, 'rb') as f:
+                    blob = f.read()
+                encoding = helper.guess_encoding(blob)
+                logging.debug('template encoding: `%s`' % encoding)
+                self.envelope.parse_template(blob.decode(encoding))
             except Exception as e:
                 ui.notify(str(e), priority='error')
                 return
@@ -801,6 +793,8 @@ class ComposeCommand(Command):
             self.envelope.add('Cc', ','.join(self.cc))
         if self.bcc:
             self.envelope.add('Bcc', ','.join(self.bcc))
+        if self.tags:
+            self.envelope.tags = [t for t in self.tags.split(',') if t]
 
         # get missing From header
         if 'From' not in self.envelope.headers:
@@ -822,7 +816,14 @@ class ComposeCommand(Command):
         # find out the right account
         sender = self.envelope.get('From')
         name, addr = email.utils.parseaddr(sender)
-        account = settings.get_account_by_address(addr)
+        try:
+            account = settings.get_account_by_address(addr)
+        except NoMatchingAccount:
+            msg = 'Cannot compose mail - no account found for `%s`' % addr
+            logging.error(msg)
+            ui.notify(msg, priority='error')
+            raise CommandCanceled()
+
         if account is None:
             accounts = settings.get_accounts()
             if not accounts:
@@ -862,8 +863,9 @@ class ComposeCommand(Command):
                 self.envelope.sign = account.sign_by_default
                 self.envelope.sign_key = account.gpg_key
             else:
-                msg = 'Cannot find gpg key for account {}'.format(account.address)
-                logging.warn(msg)
+                msg = 'Cannot find gpg key for account {}'
+                msg = msg.format(account.address)
+                logging.warning(msg)
                 ui.notify(msg, priority='error')
 
         # get missing To header
@@ -894,7 +896,8 @@ class ComposeCommand(Command):
 
         if settings.get('compose_ask_tags'):
             comp = TagsCompleter(ui.dbman)
-            tagsstring = yield ui.prompt('Tags', completer=comp)
+            tags = ','.join(self.tags) if self.tags else ''
+            tagsstring = yield ui.prompt('Tags', text=tags, completer=comp)
             tags = [t for t in tagsstring.split(',') if t]
             if tags is None:
                 raise CommandCanceled()
@@ -917,7 +920,8 @@ class ComposeCommand(Command):
             logging.debug("Trying to encrypt message because "
                           "account.encrypt_by_default=%s",
                           account.encrypt_by_default)
-            yield set_encrypt(ui, self.envelope, block_error=self.encrypt, signed_only=True)
+            yield set_encrypt(ui, self.envelope, block_error=self.encrypt,
+                              signed_only=True)
         else:
             logging.debug("No encryption by default, encrypt_by_default=%s",
                           account.encrypt_by_default)
@@ -969,4 +973,8 @@ class ReloadCommand(Command):
     """Reload configuration."""
 
     def apply(self, ui):
-        settings.reload()
+        try:
+            settings.reload()
+        except ConfigError as e:
+            ui.notify('Error when reloading config files:\n {}'.format(e),
+                      priority='error')
