@@ -14,7 +14,7 @@ from email.utils import getaddresses, parseaddr, formataddr
 from email.message import Message
 
 from twisted.internet.defer import inlineCallbacks
-from cStringIO import StringIO
+from io import BytesIO
 
 from . import Command, registerCommand
 from .globals import ExternalCommand
@@ -71,11 +71,13 @@ def determine_sender(mail, action='reply'):
         # pick the most important account that has an address in candidates
         # and use that accounts realname and the address found here
         for account in my_accounts:
-            acc_addresses = [re.escape(a) for a in account.get_addresses()]
+            acc_addresses = [re.escape(unicode(a)) for a in account.get_addresses()]
             if account.alias_regexp is not None:
                 acc_addresses.append(account.alias_regexp)
             for alias in acc_addresses:
-                regex = re.compile('^' + alias + '$', flags=re.IGNORECASE)
+                regex = re.compile(
+                    u'^' + unicode(alias) + u'$',
+                    flags=re.IGNORECASE if not account.address.case_sensitive else 0)
                 for seen_name, seen_address in candidate_addresses:
                     if regex.match(seen_address):
                         logging.debug("match!: '%s' '%s'", seen_address, alias)
@@ -186,7 +188,7 @@ class ReplyCommand(Command):
         try:
             from_header, _ = determine_sender(mail, 'reply')
         except AssertionError as e:
-            ui.notify(e.message, priority='error')
+            ui.notify(str(e), priority='error')
             return
         envelope.add('From', from_header)
 
@@ -394,7 +396,7 @@ class ForwardCommand(Command):
         try:
             from_header, _ = determine_sender(mail, 'reply')
         except AssertionError as e:
-            ui.notify(e.message, priority='error')
+            ui.notify(str(e), priority='error')
             return
         envelope.add('From', from_header)
 
@@ -441,7 +443,7 @@ class BounceMailCommand(Command):
         try:
             resent_from_header, account = determine_sender(mail, 'bounce')
         except AssertionError as e:
-            ui.notify(e.message, priority='error')
+            ui.notify(str(e), priority='error')
             return
         mail['Resent-From'] = resent_from_header
 
@@ -533,13 +535,17 @@ class EditNewCommand(Command):
     forced={'all_headers': 'toggle'},
     arguments=[(['query'], {'help': 'query used to filter messages to affect',
                             'nargs': '*'})])
+@registerCommand(
+    MODE, 'indent', help='change message/reply indentation',
+    arguments=[(['indent'], {'action': cargparse.ValidatedStoreAction,
+                             'validator': cargparse.is_int_or_pm})])
 class ChangeDisplaymodeCommand(Command):
 
     """fold or unfold messages"""
     repeatable = True
 
     def __init__(self, query=None, visible=None, raw=None, all_headers=None,
-                 **kwargs):
+                 indent=None, **kwargs):
         """
         :param query: notmuch query string used to filter messages to affect
         :type query: str
@@ -549,6 +555,8 @@ class ChangeDisplaymodeCommand(Command):
         :type raw: True, False, 'toggle' or None
         :param all_headers: show all headers (only visible if not in raw mode)
         :type all_headers: True, False, 'toggle' or None
+        :param indent: message/reply indentation
+        :type indent: '+', '-', or int
         """
         self.query = None
         if query:
@@ -556,10 +564,28 @@ class ChangeDisplaymodeCommand(Command):
         self.visible = visible
         self.raw = raw
         self.all_headers = all_headers
+        self.indent = indent
         Command.__init__(self, **kwargs)
 
     def apply(self, ui):
         tbuffer = ui.current_buffer
+
+        # set message/reply indentation if changed
+        if self.indent is not None:
+            if self.indent == '+':
+                newindent = tbuffer._indent_width + 1
+            elif self.indent == '-':
+                newindent = tbuffer._indent_width - 1
+            else:
+                # argparse validation guarantees that self.indent
+                # can be cast to an integer
+                newindent = int(self.indent)
+            # make sure indent remains non-negative
+            tbuffer._indent_width = max(newindent, 0)
+            tbuffer.rebuild()
+            tbuffer.collapse_all()
+            ui.update()
+
         logging.debug('matching lines %s...', self.query)
         if self.query is None:
             messagetrees = [tbuffer.get_selected_messagetree()]
@@ -731,7 +757,7 @@ class PipeCommand(Command):
         if self.shell:
             self.cmd = [' '.join(self.cmd)]
 
-        # do teh monkey
+        # do the monkey
         for mail in pipestrings:
             if self.background:
                 logging.debug('call in background: %s', self.cmd)
@@ -743,18 +769,15 @@ class PipeCommand(Command):
                 if self.notify_stdout:
                     ui.notify(out)
             else:
-                logging.debug('stop urwid screen')
-                ui.mainloop.screen.stop()
-                logging.debug('call: %s', self.cmd)
-                # if proc.stdout is defined later calls to communicate
-                # seem to be non-blocking!
-                proc = subprocess.Popen(self.cmd, shell=True,
-                                        stdin=subprocess.PIPE,
-                                        # stdout=subprocess.PIPE,
-                                        stderr=subprocess.PIPE)
-                out, err = proc.communicate(mail)
-                logging.debug('start urwid screen')
-                ui.mainloop.screen.start()
+                with ui.paused():
+                    logging.debug('call: %s', self.cmd)
+                    # if proc.stdout is defined later calls to communicate
+                    # seem to be non-blocking!
+                    proc = subprocess.Popen(self.cmd, shell=True,
+                                            stdin=subprocess.PIPE,
+                                            # stdout=subprocess.PIPE,
+                                            stderr=subprocess.PIPE)
+                    out, err = proc.communicate(mail)
             if err:
                 ui.notify(err, priority='error')
                 return
@@ -970,7 +993,7 @@ class OpenAttachmentCommand(Command):
                 def afterwards():
                     os.unlink(tempfile_name)
             else:
-                handler_stdin = StringIO()
+                handler_stdin = BytesIO()
                 self.attachment.write(handler_stdin)
 
             # create handler command list
@@ -1108,7 +1131,7 @@ class TagCommand(Command):
         :type action: str
         :param all: tag all messages in thread
         :type all: bool
-        :param flush: imediately write out to the index
+        :param flush: immediately write out to the index
         :type flush: bool
         """
         self.tagsstring = tags
