@@ -75,7 +75,7 @@ class StringlistCompleter(Completer):
         re_prefix = '.*' if self.match_anywhere else ''
 
         def match(s, m):
-            r = re_prefix + m + '.*'
+            r = '{}{}.*'.format(re_prefix, re.escape(m))
             return re.match(r, s, flags=self.flags) is not None
 
         return [(a, len(a)) for a in self.resultlist if match(a, pref)]
@@ -129,7 +129,7 @@ class QueryCompleter(Completer):
     """completion for a notmuch query string"""
     def __init__(self, dbman):
         """
-        :param dbman: used to look up avaliable tagstrings
+        :param dbman: used to look up available tagstrings
         :type dbman: :class:`~alot.db.DBManager`
         """
         self.dbman = dbman
@@ -172,7 +172,7 @@ class TagCompleter(StringlistCompleter):
 
     def __init__(self, dbman):
         """
-        :param dbman: used to look up avaliable tagstrings
+        :param dbman: used to look up available tagstrings
         :type dbman: :class:`~alot.db.DBManager`
         """
         resultlist = dbman.get_all_tags()
@@ -184,7 +184,7 @@ class TagsCompleter(MultipleSelectionCompleter):
 
     def __init__(self, dbman):
         """
-        :param dbman: used to look up avaliable tagstrings
+        :param dbman: used to look up available tagstrings
         :type dbman: :class:`~alot.db.DBManager`
         """
         self._completer = TagCompleter(dbman)
@@ -308,7 +308,7 @@ class CommandCompleter(Completer):
 
     def __init__(self, dbman, mode, currentbuffer=None):
         """
-        :param dbman: used to look up avaliable tagstrings
+        :param dbman: used to look up available tagstrings
         :type dbman: :class:`~alot.db.DBManager`
         :param mode: mode identifier
         :type mode: str
@@ -437,10 +437,10 @@ class CommandCompleter(Completer):
                                                                    localpos)
 
                         # prepend 'set ' + header and correct position
-                        def f((completed, pos)):
+                        def f(completed, pos):
                             return ('%s %s' % (header, completed),
                                     pos + len(header) + 1)
-                        res = [f(r) for r in res]
+                        res = [f(c, p) for c, p in res]
                         logging.debug(res)
 
                 elif self.mode == 'envelope' and cmd == 'unset':
@@ -508,7 +508,7 @@ class CommandLineCompleter(Completer):
 
     def __init__(self, dbman, mode, currentbuffer=None):
         """
-        :param dbman: used to look up avaliable tagstrings
+        :param dbman: used to look up available tagstrings
         :type dbman: :class:`~alot.db.DBManager`
         :param mode: mode identifier
         :type mode: str
