@@ -18,6 +18,11 @@ from ..settings.const import settings
 
 charset.add_charset('utf-8', charset.QP, charset.QP, 'utf-8')
 
+MISSING_HTML_MSG = ("This message contains a text/html part that was not "
+                    "rendered due to a missing mailcap entry. "
+                    "Please refer to item 5 in our FAQ: "
+                    "http://alot.rtfd.io/en/latest/faq.html")
+
 
 @functools.total_ordering
 class Message(object):
@@ -45,14 +50,24 @@ class Message(object):
         except ValueError:
             self._datetime = None
         self._filename = msg.get_filename()
-        try:
-            self._from = decode_header(msg.get_header('From'))
-        except NullPointerError:
-            self._from = ''
         self._email = None  # will be read upon first use
         self._attachments = None  # will be read upon first use
         self._tags = set(msg.get_tags())
 
+        try:
+            sender = decode_header(msg.get_header('From'))
+            if not sender:
+                sender = decode_header(msg.get_header('Sender'))
+        except NullPointerError:
+            sender = None
+        if sender:
+            self._from = sender
+        elif 'draft' in self._tags:
+            acc = settings.get_accounts()[0]
+            self._from = '"{}" <{}>'.format(acc.realname, unicode(acc.address))
+        else:
+            self._from = '"Unknown" <>'
+
     def __str__(self):
         """prettyprint the message"""
         aname, aaddress = self.get_author()
@@ -66,17 +81,17 @@ class Message(object):
 
     def __eq__(self, other):
         if isinstance(other, type(self)):
-            return self.get_message_id() == other.get_message_id()
+            return self._id == other.get_message_id()
         return NotImplemented
 
     def __ne__(self, other):
         if isinstance(other, type(self)):
-            return self.get_message_id() != other.get_message_id()
+            return self._id != other.get_message_id()
         return NotImplemented
 
     def __lt__(self, other):
         if isinstance(other, type(self)):
-            return self.get_message_id() < other.get_message_id()
+            return self._id < other.get_message_id()
         return NotImplemented
 
     def get_email(self):
@@ -116,8 +131,7 @@ class Message(object):
 
     def get_tags(self):
         """returns tags attached to this message as list of strings"""
-        l = sorted(self._tags)
-        return l
+        return sorted(self._tags)
 
     def get_thread(self):
         """returns the :class:`~alot.db.Thread` this msg belongs to"""
@@ -247,7 +261,14 @@ class Message(object):
         returns bodystring extracted from this mail
         """
         # TODO: allow toggle commands to decide which part is considered body
-        return extract_body(self.get_email())
+        eml = self.get_email()
+        bodytext = extract_body(eml)
+
+        # check if extracted body is empty but msg contains html parts
+        if (not bodytext and
+           'text/html' in (part.get_content_type() for part in eml.walk())):
+            return MISSING_HTML_MSG
+        return bodytext
 
     def get_text_content(self):
         return extract_body(self.get_email(), types=['text/plain'])
