@@ -15,7 +15,7 @@ from configobj import ConfigObj, Section
 from ..account import SendmailAccount
 from ..addressbook.abook import AbookAddressBook
 from ..addressbook.external import ExternalAddressbook
-from ..helper import pretty_datetime, string_decode
+from ..helper import pretty_datetime, string_decode, get_xdg_env
 from ..utils import configobj as checks
 
 from .errors import ConfigError, NoMatchingAccount
@@ -25,7 +25,8 @@ from .theme import Theme
 
 
 DEFAULTSPATH = os.path.join(os.path.dirname(__file__), '..', 'defaults')
-DATA_DIRS = os.environ.get('XDG_DATA_DIRS', '/usr/local/share:/usr/share').split(':')
+DATA_DIRS = get_xdg_env('XDG_DATA_DIRS',
+                        '/usr/local/share:/usr/share').split(':')
 
 
 class SettingsManager(object):
@@ -37,8 +38,10 @@ class SettingsManager(object):
         :param notmuch_rc: path to notmuch's config file
         :type notmuch_rc: str
         """
-        assert alot_rc is None or (isinstance(alot_rc, basestring) and os.path.exists(alot_rc))
-        assert notmuch_rc is None or (isinstance(notmuch_rc, basestring) and os.path.exists(notmuch_rc))
+        assert alot_rc is None or (isinstance(alot_rc, basestring) and
+                                   os.path.exists(alot_rc))
+        assert notmuch_rc is None or (isinstance(notmuch_rc, basestring) and
+                                      os.path.exists(notmuch_rc))
         self.hooks = None
         self._mailcaps = mailcap.getcaps()
         self._notmuchconfig = None
@@ -62,7 +65,8 @@ class SettingsManager(object):
         Implementation Detail: this is the same code called by the constructor
         to set bindings at alot startup.
         """
-        self._bindings = ConfigObj(os.path.join(DEFAULTSPATH, 'default.bindings'))
+        self._bindings = ConfigObj(os.path.join(DEFAULTSPATH,
+                                                'default.bindings'))
         self.read_notmuch_config()
         self.read_config()
 
@@ -75,13 +79,14 @@ class SettingsManager(object):
         """parse alot's config file from path"""
         spec = os.path.join(DEFAULTSPATH, 'alot.rc.spec')
         newconfig = read_config(
-            self.alot_rc_path, spec, checks={
+            self.alot_rc_path, spec, report_extra=True, checks={
                 'mail_container': checks.mail_container,
                 'force_list': checks.force_list,
                 'align': checks.align_mode,
                 'attrtriple': checks.attr_triple,
                 'gpg_key_hint': checks.gpg_key})
         self._config.merge(newconfig)
+        self._config.walk(self._expand_config_values)
 
         hooks_path = os.path.expanduser(self._config.get('hooksfile'))
         try:
@@ -92,16 +97,14 @@ class SettingsManager(object):
             newbindings = newconfig['bindings']
             if isinstance(newbindings, Section):
                 self._bindings.merge(newbindings)
+
+        tempdir = self._config.get('template_dir')
+        logging.debug('template directory: `%s`' % tempdir)
+
         # themes
         themestring = newconfig['theme']
         themes_dir = self._config.get('themes_dir')
-        if themes_dir:
-            themes_dir = os.path.expanduser(themes_dir)
-        else:
-            configdir = os.environ.get('XDG_CONFIG_HOME',
-                                       os.path.expanduser('~/.config'))
-            themes_dir = os.path.join(configdir, 'alot', 'themes')
-        logging.debug(themes_dir)
+        logging.debug('themes directory: `%s`' % themes_dir)
 
         # if config contains theme string use that
         data_dirs = [os.path.join(d, 'alot/themes') for d in DATA_DIRS]
@@ -112,22 +115,19 @@ class SettingsManager(object):
             # tl/dr; If the loop loads a theme it breaks. If it doesn't break,
             # then it raises a ConfigError.
             for dir_ in itertools.chain([themes_dir], data_dirs):
-                if not os.path.isdir(dir_):
-                    logging.warning(
-                        'cannot find theme %s: themes_dir %s is missing',
-                        themestring, dir_)
+                theme_path = os.path.join(dir_, themestring)
+                if not os.path.exists(os.path.expanduser(theme_path)):
+                    logging.warning('Theme `%s` does not exist.', theme_path)
                 else:
-                    theme_path = os.path.join(dir_, themestring)
                     try:
                         self._theme = Theme(theme_path)
                     except ConfigError as e:
-                        logging.warning(
-                            'Theme file %s failed validation: %s',
-                            themestring, str(e.message))
+                        raise ConfigError('Theme file `%s` failed '
+                                          'validation:\n%s' % (theme_path, e))
                     else:
                         break
             else:
-                raise ConfigError('Cannot load theme {}, see log for more '
+                raise ConfigError('Could not find theme {}, see log for more '
                                   'information'.format(themestring))
 
         # if still no theme is set, resort to default
@@ -138,6 +138,55 @@ class SettingsManager(object):
         self._accounts = self._parse_accounts(self._config)
         self._accountmap = self._account_table(self._accounts)
 
+    @staticmethod
+    def _expand_config_values(section, key):
+        """
+        Walker function for ConfigObj.walk
+
+        Applies expand_environment_and_home to all configuration values that
+        are strings (or strings that are elements of tuples/lists)
+
+        :param section: as passed by ConfigObj.walk
+        :param key: as passed by ConfigObj.walk
+        """
+
+        def expand_environment_and_home(value):
+            """
+            Expands environment variables and the home directory (~).
+
+            $FOO and ${FOO}-style environment variables are expanded, if they
+            exist. If they do not exist, they are left unchanged.
+            The exception are the following $XDG_* variables that are
+            expanded to fallback values, if they are empty or not set:
+            $XDG_CONFIG_HOME
+            $XDG_CACHE_HOME
+
+            :param value: configuration string
+            :type value: str
+            """
+            xdg_vars = {'XDG_CONFIG_HOME': '~/.config',
+                        'XDG_CACHE_HOME': '~/.cache'}
+
+            for xdg_name, fallback in xdg_vars.items():
+                if xdg_name in value:
+                    xdg_value = get_xdg_env(xdg_name, fallback)
+                    value = value.replace('$%s' % xdg_name, xdg_value)\
+                                 .replace('${%s}' % xdg_name, xdg_value)
+            return os.path.expanduser(os.path.expandvars(value))
+
+        value = section[key]
+
+        if isinstance(value, (str, unicode)):
+            section[key] = expand_environment_and_home(value)
+        elif isinstance(value, (list, tuple)):
+            new = list()
+            for item in value:
+                if isinstance(item, (str, unicode)):
+                    new.append(expand_environment_and_home(item))
+                else:
+                    new.append(item)
+            section[key] = new
+
     @staticmethod
     def _parse_accounts(config):
         """
@@ -316,7 +365,7 @@ class SettingsManager(object):
         fallback_focus = resolve_att(onebelow_focus, default_focus)
 
         for sec in cfg['tags'].sections:
-            if re.match('^' + sec + '$', tag):
+            if re.match('^{}$'.format(re.escape(sec)), tag):
                 normal = resolve_att(colourpick(cfg['tags'][sec]['normal']),
                                      fallback_normal)
                 focus = resolve_att(colourpick(cfg['tags'][sec]['focus']),
@@ -434,7 +483,7 @@ class SettingsManager(object):
             the default account wil be returned
         :rtype: :class:`Account`
         :raises ~alot.settings.errors.NoMatchingAccount: If no account can be
-            found. Thsi includes if return_default is True and there are no
+            found. This includes if return_default is True and there are no
             accounts defined.
         """
         _, address = email.utils.parseaddr(address)
