@@ -6,6 +6,8 @@ from __future__ import absolute_import
 import logging
 import os
 import signal
+import codecs
+import contextlib
 
 from twisted.internet import reactor, defer, task
 import urwid
@@ -18,6 +20,7 @@ from .commands import CommandCanceled
 from .commands import CommandParseError
 from .helper import split_commandline
 from .helper import string_decode
+from .helper import get_xdg_env
 from .widgets.globals import CompleteEdit
 from .widgets.globals import ChoiceWidget
 
@@ -73,6 +76,9 @@ class UI(object):
         # alarm handle for callback that clears input queue (to cancel alarm)
         self._alarm = None
 
+        # force urwid to pass key events as unicode, independent of LANG
+        urwid.set_encoding('utf-8')
+
         # create root widget
         global_att = settings.get_theming_attribute('global', 'body')
         mainframe = urwid.Frame(urwid.SolidFill())
@@ -83,7 +89,7 @@ class UI(object):
 
         # load histories
         self._cache = os.path.join(
-            os.environ.get('XDG_CACHE_HOME', os.path.expanduser('~/.cache')),
+            get_xdg_env('XDG_CACHE_HOME', os.path.expanduser('~/.cache')),
             'alot', 'history')
         self._cmd_hist_file = os.path.join(self._cache, 'commands')
         self._sender_hist_file = os.path.join(self._cache, 'senders')
@@ -174,7 +180,7 @@ class UI(object):
                     try:
                         self.apply_commandline(cmdline)
                     except CommandParseError as e:
-                        self.notify(e.message, priority='error')
+                        self.notify(str(e), priority='error')
                 # move keys are always passed
                 elif cmdline in ['move up', 'move down', 'move page up',
                                  'move page down']:
@@ -262,7 +268,7 @@ class UI(object):
         """
         Called by :class:`urwid.MainLoop` if a keypress was passed to the root
         widget by `self._input_filter` but is not handled in any widget. We
-        keep it for debuging purposes.
+        keep it for debugging purposes.
         """
         logging.debug('unhandled input: %s', key)
 
@@ -358,6 +364,24 @@ class UI(object):
             exit_msg = 'Could not stop reactor: {}.'.format(e)
             logging.error('%s\nShutting down anyway..', exit_msg)
 
+    @contextlib.contextmanager
+    def paused(self):
+        """
+        context manager that pauses the UI to allow running external commands.
+
+        If an exception occurs, the UI will be started before the exception is
+        re-raised.
+        """
+        self.mainloop.stop()
+        try:
+            yield
+        finally:
+            self.mainloop.start()
+
+            # make sure urwid renders its canvas at the correct size
+            self.mainloop.screen_size = None
+            self.mainloop.draw_screen()
+
     def buffer_open(self, buf):
         """register and focus new :class:`~alot.buffers.Buffer`."""
 
@@ -482,7 +506,7 @@ class UI(object):
         self.update()
 
     def choice(self, message, choices=None, select=None, cancel=None,
-               msg_position='above'):
+               msg_position='above', choices_to_return=None):
         """
         prompt user to make a choice.
 
@@ -490,6 +514,9 @@ class UI(object):
         :type message: unicode
         :param choices: dict of possible choices
         :type choices: dict: keymap->choice (both str)
+        :param choices_to_return: dict of possible choices to return for the
+                                  choices of the choices of paramter
+        :type choices: dict: keymap->choice key is  str and value is any obj)
         :param select: choice to return if enter/return is hit. Ignored if set
                        to `None`.
         :type select: str
@@ -502,6 +529,7 @@ class UI(object):
         :rtype:  :class:`twisted.defer.Deferred`
         """
         choices = choices or {'y': 'yes', 'n': 'no'}
+        choices_to_return = choices_to_return or {}
         assert select is None or select in choices.itervalues()
         assert cancel is None or cancel in choices.itervalues()
         assert msg_position in ['left', 'above']
@@ -518,8 +546,10 @@ class UI(object):
 
         # set up widgets
         msgpart = urwid.Text(message)
-        choicespart = ChoiceWidget(choices, callback=select_or_cancel,
-                                   select=select, cancel=cancel)
+        choicespart = ChoiceWidget(choices,
+                                   choices_to_return=choices_to_return,
+                                   callback=select_or_cancel, select=select,
+                                   cancel=cancel)
 
         # build widget
         if msg_position == 'left':
@@ -729,7 +759,7 @@ class UI(object):
         if size == 0:
             return []
         if os.path.exists(path):
-            with open(path) as histfile:
+            with codecs.open(path, 'r', encoding='utf-8') as histfile:
                 lines = [line.rstrip('\n') for line in histfile]
             if size > 0:
                 lines = lines[-size:]
@@ -759,7 +789,7 @@ class UI(object):
         if not os.path.exists(directory):
             os.makedirs(directory)
         # Write linewise to avoid building a large string in menory.
-        with open(path, 'w') as histfile:
+        with codecs.open(path, 'w', encoding='utf-8') as histfile:
             for line in history:
                 histfile.write(line)
                 histfile.write('\n')
